@@ -3,6 +3,7 @@
  * Copyright (C) 2001-2003, Ximian, Inc.
  */
 
+#include <dirent.h>
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -10,7 +11,6 @@
 #include <sys/stat.h>
 
 #include <libsoup/soup.h>
-#include <glib/gstdio.h>
 
 static int
 compare_strings (gconstpointer a, gconstpointer b)
@@ -27,22 +27,22 @@ get_directory_listing (const char *path)
 	GPtrArray *entries;
 	GString *listing;
 	char *escaped;
-	GDir *dir;
-	const gchar *d_name;
+	DIR *dir;
+	struct dirent *dent;
 	int i;
 
 	entries = g_ptr_array_new ();
-	dir = g_dir_open (path, 0, NULL);
+	dir = opendir (path);
 	if (dir) {
-		while ((d_name = g_dir_read_name (dir))) {
-			if (!strcmp (d_name, ".") ||
-			    (!strcmp (d_name, "..") &&
+		while ((dent = readdir (dir))) {
+			if (!strcmp (dent->d_name, ".") ||
+			    (!strcmp (dent->d_name, "..") &&
 			     !strcmp (path, "./")))
 				continue;
-			escaped = g_markup_escape_text (d_name, -1);
+			escaped = g_markup_escape_text (dent->d_name, -1);
 			g_ptr_array_add (entries, escaped);
 		}
-		g_dir_close (dir);
+		closedir (dir);
 	}
 
 	g_ptr_array_sort (entries, (GCompareFunc)compare_strings);
@@ -68,24 +68,19 @@ static void
 do_get (SoupServer *server, SoupMessage *msg, const char *path)
 {
 	char *slash;
-	GStatBuf st;
+	struct stat st;
 
-	if (g_stat (path, &st) == -1) {
+	if (stat (path, &st) == -1) {
 		if (errno == EPERM)
 			soup_message_set_status (msg, SOUP_STATUS_FORBIDDEN);
 		else if (errno == ENOENT)
 			soup_message_set_status (msg, SOUP_STATUS_NOT_FOUND);
 		else
 			soup_message_set_status (msg, SOUP_STATUS_INTERNAL_SERVER_ERROR);
-		/* In a real program you should also provide some text explaining the
-		 * error to the user (via soup_message_set_response), and include in it
-		 * the request path, as otherwise some browsers (Firefox, WebKit) only
-		 * show a blank page, confusing the user.
-		 */
 		return;
 	}
 
-	if (g_file_test (path, G_FILE_TEST_IS_DIR)) {
+	if (S_ISDIR (st.st_mode)) {
 		GString *listing;
 		char *index_path;
 
@@ -101,7 +96,7 @@ do_get (SoupServer *server, SoupMessage *msg, const char *path)
 		}
 
 		index_path = g_strdup_printf ("%s/index.html", path);
-		if (g_stat (index_path, &st) != -1) {
+		if (stat (index_path, &st) != -1) {
 			do_get (server, msg, index_path);
 			g_free (index_path);
 			return;
@@ -112,7 +107,6 @@ do_get (SoupServer *server, SoupMessage *msg, const char *path)
 		soup_message_set_response (msg, "text/html",
 					   SOUP_MEMORY_TAKE,
 					   listing->str, listing->len);
-		soup_message_set_status (msg, SOUP_STATUS_OK);
 		g_string_free (listing, FALSE);
 		return;
 	}
@@ -151,18 +145,18 @@ do_get (SoupServer *server, SoupMessage *msg, const char *path)
 static void
 do_put (SoupServer *server, SoupMessage *msg, const char *path)
 {
-	GStatBuf st;
+	struct stat st;
 	FILE *f;
 	gboolean created = TRUE;
 
-	if (g_stat (path, &st) != -1) {
+	if (stat (path, &st) != -1) {
 		const char *match = soup_message_headers_get_one (msg->request_headers, "If-None-Match");
 		if (match && !strcmp (match, "*")) {
 			soup_message_set_status (msg, SOUP_STATUS_CONFLICT);
 			return;
 		}
 
-		if (!g_file_test (path, G_FILE_TEST_IS_REGULAR)) {
+		if (!S_ISREG (st.st_mode)) {
 			soup_message_set_status (msg, SOUP_STATUS_FORBIDDEN);
 			return;
 		}
@@ -219,19 +213,22 @@ quit (int sig)
 	exit (0);
 }
 
-static int port;
-static const char *tls_cert_file, *tls_key_file;
+static int port, ssl_port;
+static const char *ssl_cert_file, *ssl_key_file;
 
 static GOptionEntry entries[] = {
 	{ "cert-file", 'c', 0,
-	  G_OPTION_ARG_STRING, &tls_cert_file,
+	  G_OPTION_ARG_STRING, &ssl_cert_file,
 	  "Use FILE as the TLS certificate file", "FILE" },
 	{ "key-file", 'k', 0,
-	  G_OPTION_ARG_STRING, &tls_key_file,
+	  G_OPTION_ARG_STRING, &ssl_key_file,
 	  "Use FILE as the TLS private key file", "FILE" },
 	{ "port", 'p', 0,
 	  G_OPTION_ARG_INT, &port,
 	  "Port to listen on", NULL },
+	{ "ssl-port", 's', 0,
+	  G_OPTION_ARG_INT, &port,
+	  "Port to listen on for TLS traffic", NULL },
 	{ NULL }
 };
 
@@ -240,10 +237,7 @@ main (int argc, char **argv)
 {
 	GOptionContext *opts;
 	GMainLoop *loop;
-	SoupServer *server;
-	GSList *uris, *u;
-	char *str;
-	GTlsCertificate *cert;
+	SoupServer *server, *ssl_server;
 	GError *error = NULL;
 
 	opts = g_option_context_new (NULL);
@@ -264,35 +258,36 @@ main (int argc, char **argv)
 
 	signal (SIGINT, quit);
 
-	if (tls_cert_file && tls_key_file) {
-		cert = g_tls_certificate_new_from_files (tls_cert_file, tls_key_file, &error);
-		if (error) {
-			g_printerr ("Unable to create server: %s\n", error->message);
-			exit (1);
-		}
-		server = soup_server_new (SOUP_SERVER_SERVER_HEADER, "simple-httpd ",
-					  SOUP_SERVER_TLS_CERTIFICATE, cert,
-					  NULL);
-		g_object_unref (cert);
-
-		soup_server_listen_all (server, port, SOUP_SERVER_LISTEN_HTTPS, &error);
-	} else {
-		server = soup_server_new (SOUP_SERVER_SERVER_HEADER, "simple-httpd ",
-					  NULL);
-		soup_server_listen_all (server, port, 0, &error);
+	server = soup_server_new (SOUP_SERVER_PORT, port,
+				  SOUP_SERVER_SERVER_HEADER, "simple-httpd ",
+				  NULL);
+	if (!server) {
+		g_printerr ("Unable to bind to server port %d\n", port);
+		exit (1);
 	}
-
 	soup_server_add_handler (server, NULL,
 				 server_callback, NULL, NULL);
+	g_print ("\nStarting Server on port %d\n",
+		 soup_server_get_port (server));
+	soup_server_run_async (server);
 
-	uris = soup_server_get_uris (server);
-	for (u = uris; u; u = u->next) {
-		str = soup_uri_to_string (u->data, FALSE);
-		g_print ("Listening on %s\n", str);
-		g_free (str);
-		soup_uri_free (u->data);
+	if (ssl_cert_file && ssl_key_file) {
+		ssl_server = soup_server_new (
+			SOUP_SERVER_PORT, ssl_port,
+			SOUP_SERVER_SSL_CERT_FILE, ssl_cert_file,
+			SOUP_SERVER_SSL_KEY_FILE, ssl_key_file,
+			NULL);
+
+		if (!ssl_server) {
+			g_printerr ("Unable to bind to SSL server port %d\n", ssl_port);
+			exit (1);
+		}
+		soup_server_add_handler (ssl_server, NULL,
+					 server_callback, NULL, NULL);
+		g_print ("Starting SSL Server on port %d\n", 
+			 soup_server_get_port (ssl_server));
+		soup_server_run_async (ssl_server);
 	}
-	g_slist_free (uris);
 
 	g_print ("\nWaiting for requests...\n");
 
