@@ -8,7 +8,9 @@
 
 #include <sys/types.h>
 
+#ifndef SOUP_COMPILATION
 #include <libsoup/soup-portability.h>
+#endif
 #include <libsoup/soup-types.h>
 
 G_BEGIN_DECLS
@@ -51,6 +53,8 @@ typedef enum {
 
 #define SOUP_ADDRESS_ANY_PORT 0
 
+struct sockaddr;
+
 typedef void   (*SoupAddressCallback)            (SoupAddress         *addr,
 						  guint                status,
 						  gpointer             user_data);
