@@ -503,7 +503,7 @@ io_read (SoupMessage *msg, GCancellable *cancellable, GError **error)
 		status = io->parse_headers_cb (msg, (char *)io->read_header_buf->data,
 					       io->read_header_buf->len,
 					       &io->read_encoding,
-					       io->header_data);
+					       io->header_data, error);
 		g_byte_array_set_size (io->read_header_buf, 0);
 
 		if (status != SOUP_STATUS_OK) {
@@ -1134,6 +1134,7 @@ soup_message_io_pause (SoupMessage *msg)
 
 	if (io->unpause_source) {
 		g_source_destroy (io->unpause_source);
+		g_source_unref (io->unpause_source);
 		io->unpause_source = NULL;
 	}
 
@@ -1147,6 +1148,8 @@ io_unpause_internal (gpointer msg)
 	SoupMessageIOData *io = priv->io_data;
 
 	g_return_val_if_fail (io != NULL, FALSE);
+
+	g_source_unref (io->unpause_source);
 	io->unpause_source = NULL;
 	io->paused = FALSE;
 
@@ -1173,7 +1176,7 @@ soup_message_io_unpause (SoupMessage *msg)
 
 	if (!io->blocking) {
 		if (!io->unpause_source) {
-			io->unpause_source = soup_add_completion (
+			io->unpause_source = soup_add_completion_reffed (
 				io->async_context, io_unpause_internal, msg);
 		}
 	} else
