@@ -5,9 +5,6 @@
  * Copyright (C) 2000-2003, Ximian, Inc.
  */
 
-//#include <stdlib.h>
-//#include <string.h>
-
 #include "soup-message.h"
 #include "soup.h"
 #include "soup-connection.h"
@@ -1196,7 +1193,6 @@ soup_message_set_auth (SoupMessage *msg, SoupAuth *auth)
 
 	g_return_if_fail (SOUP_IS_MESSAGE (msg));
 	g_return_if_fail (auth == NULL || SOUP_IS_AUTH (auth));
-	g_return_if_fail (auth == NULL || soup_auth_is_authenticated (auth));
 
 	priv = SOUP_MESSAGE_GET_PRIVATE (msg);
 
@@ -1211,9 +1207,11 @@ soup_message_set_auth (SoupMessage *msg, SoupAuth *auth)
 
 	g_object_ref (priv->auth);
 	token = soup_auth_get_authorization (auth, msg);
-	soup_message_headers_replace (msg->request_headers,
-				      "Authorization", token);
-	g_free (token);
+	if (token) {
+		soup_message_headers_replace (msg->request_headers,
+					      "Authorization", token);
+		g_free (token);
+	}
 }
 
 SoupAuth *
@@ -1232,7 +1230,6 @@ soup_message_set_proxy_auth (SoupMessage *msg, SoupAuth *auth)
 
 	g_return_if_fail (SOUP_IS_MESSAGE (msg));
 	g_return_if_fail (auth == NULL || SOUP_IS_AUTH (auth));
-	g_return_if_fail (auth == NULL || soup_auth_is_authenticated (auth));
 
 	priv = SOUP_MESSAGE_GET_PRIVATE (msg);
 
@@ -1260,6 +1257,19 @@ soup_message_get_proxy_auth (SoupMessage *msg)
 	return SOUP_MESSAGE_GET_PRIVATE (msg)->proxy_auth;
 }
 
+SoupConnection *
+soup_message_get_connection (SoupMessage *msg)
+{
+	return SOUP_MESSAGE_GET_PRIVATE (msg)->connection;
+}
+
+void
+soup_message_set_connection (SoupMessage    *msg,
+			     SoupConnection *conn)
+{
+	SOUP_MESSAGE_GET_PRIVATE (msg)->connection = conn;
+}
+
 /**
  * soup_message_cleanup_response:
  * @msg: a #SoupMessage
@@ -1623,12 +1633,13 @@ soup_message_set_status_full (SoupMessage *msg,
  * destroyed
  *
  * Sets an alternate chunk-allocation function to use when reading
- * @msg's body when using the traditional (ie, non-#SoupRequest-based)
- * API. Every time data is available to read, libsoup will call
- * @allocator, which should return a #SoupBuffer. (See
- * #SoupChunkAllocator for additional details.) Libsoup will then read
- * data from the network into that buffer, and update the buffer's
- * <literal>length</literal> to indicate how much data it read.
+ * @msg's body when using the traditional (ie,
+ * non-#SoupRequest<!-- -->-based) API. Every time data is available
+ * to read, libsoup will call @allocator, which should return a
+ * #SoupBuffer. (See #SoupChunkAllocator for additional details.)
+ * Libsoup will then read data from the network into that buffer, and
+ * update the buffer's <literal>length</literal> to indicate how much
+ * data it read.
  *
  * Generally, a custom chunk allocator would be used in conjunction
  * with soup_message_body_set_accumulate() %FALSE and
@@ -1812,11 +1823,13 @@ soup_message_set_https_status (SoupMessage *msg, SoupConnection *conn)
  * @certificate: (out) (transfer none): @msg's TLS certificate
  * @errors: (out): the verification status of @certificate
  *
- * If @msg is using https, this retrieves the #GTlsCertificate
- * associated with its connection, and the #GTlsCertificateFlags showing
- * what problems, if any, have been found with that certificate.
+ * If @msg is using https (or attempted to use https but got
+ * %SOUP_STATUS_SSL_FAILED), this retrieves the #GTlsCertificate
+ * associated with its connection, and the #GTlsCertificateFlags
+ * showing what problems, if any, have been found with that
+ * certificate.
  *
- * Return value: %TRUE if @msg uses https, %FALSE if not
+ * Return value: %TRUE if @msg used/attempted https, %FALSE if not
  *
  * Since: 2.34
  */
