@@ -9,19 +9,24 @@
 #include <config.h>
 #endif
 
-#include <string.h>
-#include <stdlib.h>
-
 #include "soup-proxy-resolver-default.h"
-#include "soup-proxy-uri-resolver.h"
-#include "soup-session-feature.h"
-#include "soup-uri.h"
+#include "soup.h"
 
 /**
- * SoupProxyResolverDefault:
+ * SECTION:soup-proxy-resolver-default
+ * @short_description: System proxy configuration integration
  *
- * A #SoupProxyURIResolver implementation that uses the default gio
- * #GProxyResolver to resolve proxies.
+ * #SoupProxyResolverDefault is a #SoupProxyURIResolver implementation
+ * that uses the default gio GProxyResolver to resolve proxies.
+ *
+ * If you are using a plain #SoupSession (ie, not #SoupSessionAsync or
+ * #SoupSessionSync), then a #SoupProxyResolverDefault will
+ * automatically be added to the session. (You can use
+ * %SOUP_SESSION_REMOVE_FEATURE_BY_TYPE at construct time if you don't
+ * want this.) If you are using one of the deprecated #SoupSession
+ * subclasses, you can add a #SoupProxyResolverDefault to your session
+ * with soup_session_add_feature() or
+ * soup_session_add_feature_by_type().
  *
  * Since: 2.34
  */
@@ -83,8 +88,7 @@ soup_proxy_resolver_default_finalize (GObject *object)
 {
 	SoupProxyResolverDefaultPrivate *priv = SOUP_PROXY_RESOLVER_DEFAULT_GET_PRIVATE (object);
 
-	if (priv->gproxy_resolver)
-		g_object_unref (priv->gproxy_resolver);
+	g_clear_object (&priv->gproxy_resolver);
 
 	G_OBJECT_CLASS (soup_proxy_resolver_default_parent_class)->finalize (object);
 }
