@@ -30,7 +30,6 @@
 #include "soup-request.h"
 #include "soup.h"
 #include "soup-requester.h"
-#include "soup-request-http.h"
 
 /**
  * SECTION:soup-request
@@ -282,6 +281,20 @@ soup_request_class_init (SoupRequestClass *request_class)
 	object_class->set_property = soup_request_set_property;
 	object_class->get_property = soup_request_get_property;
 
+	/**
+	 * SOUP_REQUEST_URI:
+	 *
+	 * Alias for the #SoupRequest:uri property, qv.
+	 *
+	 * Since: 2.42
+	 */
+	/**
+	 * SoupRequest:uri:
+	 *
+	 * The request URI.
+	 *
+	 * Since: 2.42
+	 */
 	g_object_class_install_property (
 		 object_class, PROP_URI,
 		 g_param_spec_boxed (SOUP_REQUEST_URI,
@@ -289,6 +302,20 @@ soup_request_class_init (SoupRequestClass *request_class)
 				     "The request URI",
 				     SOUP_TYPE_URI,
 				     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	/**
+	 * SOUP_REQUEST_SESSION:
+	 *
+	 * Alias for the #SoupRequest:session property, qv.
+	 *
+	 * Since: 2.42
+	 */
+	/**
+	 * SoupRequest:session:
+	 *
+	 * The request's #SoupSession.
+	 *
+	 * Since: 2.42
+	 */
 	g_object_class_install_property (
 		 object_class, PROP_SESSION,
 		 g_param_spec_object (SOUP_REQUEST_SESSION,
@@ -376,32 +403,3 @@ soup_request_get_content_type (SoupRequest  *request)
 {
 	return SOUP_REQUEST_GET_CLASS (request)->get_content_type (request);
 }
-
-/**
- * soup_request_disable_feature:
- * @request: a #SoupRequest
- * @feature_type: the #GType of a #SoupSessionFeature
- *
- * This disables the actions of #SoupSessionFeature<!-- -->s with the
- * given @feature_type (or a subclass of that type) on @request, so
- * that @request is processed as though the feature(s) hadn't been
- * added to the session. Eg, passing #SOUP_TYPE_PROXY_URI_RESOLVER for
- * @feature_type will disable proxy handling and cause @request to be
- * sent directly to the indicated origin server, regardless of system
- * proxy configuration.
- *
- * Since: 2.42
- */
-void
-soup_request_disable_feature (SoupRequest *request,
-			      GType        feature_type)
-{
-	SoupMessage *msg;
-
-	/* For now, features only affect SoupMessages, so... */
-	if (SOUP_IS_REQUEST_HTTP (request)) {
-		msg = soup_request_http_get_message (SOUP_REQUEST_HTTP (request));
-		soup_message_disable_feature (msg, feature_type);
-		g_object_unref (msg);
-	}
-}
