@@ -3,15 +3,6 @@
  * Copyright (C) 2008 Red Hat, Inc.
  */
 
-#include "config.h"
-
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-
-#include <glib.h>
-#include <libsoup/soup.h>
-
 #include "test-utils.h"
 
 typedef struct {
@@ -21,15 +12,6 @@ typedef struct {
 	gboolean streaming;
 } PutTestData;
 
-static SoupBuffer *
-error_chunk_allocator (SoupMessage *msg, gsize max_len, gpointer user_data)
-{
-	/* This should never be called, because there is no response body. */
-	debug_printf (1, "  error_chunk_allocator called!\n");
-	errors++;
-	return soup_buffer_new (SOUP_MEMORY_TAKE, g_malloc (100), 100);
-}
-
 static void
 write_next_chunk (SoupMessage *msg, gpointer user_data)
 {
@@ -191,7 +173,6 @@ do_request_test (SoupSession *session, SoupURI *base_uri, RequestTestFlags flags
 	msg = soup_message_new_from_uri ("PUT", uri);
 	soup_message_headers_set_encoding (msg->request_headers, SOUP_ENCODING_CHUNKED);
 	soup_message_body_set_accumulate (msg->request_body, FALSE);
-	soup_message_set_chunk_allocator (msg, error_chunk_allocator, NULL, NULL);
 	if (flags & HACKY_STREAMING) {
 		g_signal_connect (msg, "wrote_chunk",
 				  G_CALLBACK (write_next_chunk_streaming_hack), &ptd);
@@ -303,7 +284,9 @@ do_response_test (SoupSession *session, SoupURI *base_uri)
 
 	msg = soup_message_new_from_uri ("GET", base_uri);
 	soup_message_body_set_accumulate (msg->response_body, FALSE);
+	G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
 	soup_message_set_chunk_allocator (msg, chunk_allocator, &gtd, NULL);
+	G_GNUC_END_IGNORE_DEPRECATIONS;
 	g_signal_connect (msg, "got_chunk",
 			  G_CALLBACK (got_chunk), &gtd);
 	soup_session_send_message (session, msg);
@@ -403,6 +386,60 @@ do_temporary_test (SoupSession *session, SoupURI *base_uri)
 	g_object_unref (msg);
 }
 
+#define LARGE_CHUNK_SIZE 1000000
+
+typedef struct {
+	SoupBuffer *buf;
+	gsize offset;
+} LargeChunkData;
+
+static void
+large_wrote_body_data (SoupMessage *msg, SoupBuffer *chunk, gpointer user_data)
+{
+	LargeChunkData *lcd = user_data;
+
+	if (memcmp (chunk->data, lcd->buf->data + lcd->offset, chunk->length) != 0) {
+		debug_printf (1, "  chunk data mismatch at %ld\n", (long)lcd->offset);
+		errors++;
+	} else
+		debug_printf (2, "  chunk data match at %ld\n", (long)lcd->offset);
+	lcd->offset += chunk->length;
+}
+
+static void
+do_large_chunk_test (SoupSession *session, SoupURI *base_uri)
+{
+	SoupMessage *msg;
+	char *buf_data;
+	int i;
+	LargeChunkData lcd;
+
+	debug_printf (1, "PUT w/ large chunk\n");
+
+	msg = soup_message_new_from_uri ("PUT", base_uri);
+
+	buf_data = g_malloc0 (LARGE_CHUNK_SIZE);
+	for (i = 0; i < LARGE_CHUNK_SIZE; i++)
+		buf_data[i] = i & 0xFF;
+	lcd.buf = soup_buffer_new (SOUP_MEMORY_TAKE, buf_data, LARGE_CHUNK_SIZE);
+	lcd.offset = 0;
+	soup_message_body_append_buffer (msg->request_body, lcd.buf);
+	soup_message_body_set_accumulate (msg->request_body, FALSE);
+
+	g_signal_connect (msg, "wrote_body_data",
+			  G_CALLBACK (large_wrote_body_data), &lcd);
+	soup_session_send_message (session, msg);
+
+	if (!SOUP_STATUS_IS_SUCCESSFUL (msg->status_code)) {
+		debug_printf (1, "  message failed: %d %s\n",
+			      msg->status_code, msg->reason_phrase);
+		errors++;
+	}
+
+	soup_buffer_free (lcd.buf);
+	g_object_unref (msg);
+}
+
 static void
 do_chunk_tests (SoupURI *base_uri)
 {
@@ -422,6 +459,8 @@ do_chunk_tests (SoupURI *base_uri)
 	do_response_test (session, base_uri);
 	debug_printf (2, "\n\n");
 	do_temporary_test (session, base_uri);
+	debug_printf (2, "\n\n");
+	do_large_chunk_test (session, base_uri);
 	soup_test_session_abort_unref (session);
 }
 
@@ -434,9 +473,7 @@ server_callback (SoupServer *server, SoupMessage *msg,
 	char *md5;
 
 	if (g_str_has_prefix (path, "/redirect")) {
-		soup_message_set_status (msg, SOUP_STATUS_FOUND);
-		soup_message_headers_replace (msg->response_headers,
-					      "Location", "/");
+		soup_message_set_redirect (msg, SOUP_STATUS_FOUND, "/");
 		return;
 	}
 
