@@ -5,6 +5,8 @@
 
 #include "test-utils.h"
 
+#ifdef HAVE_PHP_XMLRPC
+
 #ifdef G_GNUC_BEGIN_IGNORE_DEPRECATIONS
 G_GNUC_BEGIN_IGNORE_DEPRECATIONS
 #endif
@@ -552,3 +554,13 @@ main (int argc, char **argv)
 	test_cleanup ();
 	return errors != 0;
 }
+
+#else /* HAVE_PHP_XMLRPC */
+
+int
+main (int argc, char **argv)
+{
+	return 77; /* SKIP */
+}
+
+#endif
